using System;
using NUnit.Framework;
using EasyMockNET;
using CreditProcessLib.Mocks;

namespace CreditProcessLib.Tests
{
	/// <summary>
	/// Test for CreditProcess
	/// </summary>
	[TestFixture]	
	public class CreditProcessTest
	{
		protected CreditProcessor processor;

		protected virtual ICardNumberValidator getCardValidator()
		{
			return new MockCardNumberValidator();
		}

		[SetUp]
		public void SetUp()
		{
			ICardNumberValidator cardValidator = getCardValidator();
			processor = new CreditProcessor(cardValidator);
		}

		public void TestCreateCreditProcessor()
		{
			Assert.IsNotNull(processor);
		}

		private const string cardNumber1 = "4255138811223344";
		private const string cardNumber2 = "0123456789";
		private const string cardNumber3 = "10234567";

		public void testCheckCardNumber()
		{
			processor.validateCardNumber(cardNumber1);
		}

		[ExpectedException(typeof(InvalidCreditCardNumberException))]
		public void testCheckInvalidCardNumber()
		{
			processor.validateCardNumber(cardNumber2);
		}

		[ExpectedException(typeof(InvalidCreditCardNumberException))]
		public void testCheckAnotherInvalidCardNumber()
		{
			processor.validateCardNumber(cardNumber3);
		}

		public void testApproveCredit()
		{
			MockControl control = MockControl.CreateControl(typeof(ICreditChargeEngine));
			ICreditChargeEngine creditCharge = control.GetMock() as ICreditChargeEngine;
			creditCharge.Charge(cardNumber1, 20);
			control.SetReturnValue(true);
			control.Replay();

			processor.SetCreditChargeEngine(creditCharge);

			int orderNumber = processor.Charge(cardNumber1, 20);

			Assert.IsTrue(orderNumber > 0);

			control.Verify();
		}

		[ExpectedException(typeof(CreditOverflowException))]
		public void testRejectCredit()
		{
			MockControl control = MockControl.CreateControl(typeof(ICreditChargeEngine));
			ICreditChargeEngine creditCharge = control.GetMock() as ICreditChargeEngine;
			creditCharge.Charge(cardNumber1, 20000);
			control.SetReturnValue(false);
			control.Replay();

			processor.SetCreditChargeEngine(creditCharge);

			try
			{
				int orderNumber = processor.Charge(cardNumber1, 20000);
			}
			finally
			{
				control.Verify();
			}
		}
	}
}
